# Copyright (C) 2004-2005 struktur AG and Contributors <simon@struktur.de>
# http://www.strukturag.com, http://www.icoya.com

# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL). A copy of the ZPL should accompany this distribution.

# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.

# See LICENSE.txt for details

""" Session Crumbler: enable session for non session user folders.

$Id: SessionCrumbler.py 11519 2005-08-30 14:51:21Z longsleep $
"""

import Globals
from ExtensionClass import Base
from AccessControl import ClassSecurityInfo

_marker=[]

class SessionSecureContainer(Base):
    """
    Provides a security wrapper around a dictionary to allow sensible data be stored into session.
    This data cannot be accessed from restricted code.
    """
    
    __store = None
    
    security = ClassSecurityInfo()
    
    def __init__(self):
        self.__store={}
    
    security.declarePrivate('get')
    def get(self, key, default=_marker):
        # get values from the store
        if default is _marker:
            return self.__store.get(key)
        else:
            return self.__store.get(key, default)
        
    security.declarePrivate('set')
    def set(self, key, value):
        # set values into the store
        self.__store[key]=value

    security.declarePrivate('items')
    def items(self):
        # returns a key, value pair for all stored items
        return self.__store.items()

    def __repr__(self):
        return "<%s instance>" % self.__class__.__name__


Globals.InitializeClass(SessionSecureContainer)
